@@ -33,8 +33,11 @@ def enum(*sequential, **named):
 
 
 HtmlPage = enum('NON_SPECIFIED', 'ONE', 'MANY', 'PPRLDMANY_BY_GROUP', 'PPRLDMANY_BY_GROUP_MANY',
-                'PPTABLE', 'PPTABLE2', 'PPTABLES', 'PPRLDISTR', 'PPRLDISTR2', 'PPLOGLOSS', 'PPSCATTER', 'PPFIGS')
+                'PPTABLE', 'PPTABLE2', 'PPTABLES', 'PPRLDISTR', 'PPRLDISTR2', 'PPLOGLOSS',
+                'PPSCATTER', 'PPFIGS', 'PPFIGCONS', 'PPFIGCONS1')
 
+_figsize_warnings = 1  # couldn't convince filterwarnings('once') to work as desired
+'''remaining number of warnings to be issued'''
 
 def save_figure(filename,
                 algorithm=None,
@@ -93,6 +96,15 @@ def save_figure(filename,
                     ' with Exception: "%s"' %
                     (plt.matplotlib.__version__, str(e)))
         try:
+            if plt.rcParams['figure.figsize'] != genericsettings.figsize:
+                # prevent saved figure to be different under Jupyter notebooks
+                plt.gcf().set_size_inches(genericsettings.figsize)
+                global _figsize_warnings
+                if _figsize_warnings > 0:
+                    m = 'Plotting with genericsettings.figsize=={} instead of default {}'.format(
+                            genericsettings.figsize, plt.rcParams['figure.figsize'])
+                    warnings.warn(m)
+                    _figsize_warnings -= 1
             plt.savefig(filename + '.' + format,
                         dpi=60 if genericsettings.in_a_hurry else 300,
                         format=format,
@@ -189,6 +201,9 @@ def save_folder_index_file(filename, image_file_extension):
     links += get_rld_link(current_dir)
     links += add_link(current_dir, None, genericsettings.ppfigdim_file_name + '.html',
                       'Scaling with dimension for selected targets')
+    links += add_link(current_dir, None, genericsettings.ppfigcons1_file_name + '.html',
+                      'Scaling with constraints for selected targets')
+
     links += add_link(current_dir, None, 'pptable.html', 'Tables for selected targets')
     links += add_link(current_dir, None, 'pprldistr.html',
                       'Runtime distribution for selected targets and f-distributions')
@@ -200,6 +215,7 @@ def save_folder_index_file(filename, image_file_extension):
         links += add_image(image_file_name, True, 380)
 
     links += add_link(current_dir, None, genericsettings.ppfigs_file_name + '.html', 'Scaling with dimension')
+    links += add_link(current_dir, None, genericsettings.ppfigcons_file_name + '.html', 'Scaling with constraints')
     links += add_link(current_dir, None, genericsettings.pptables_file_name + '.html',
                       'Tables for selected targets')
     links += add_link(current_dir, None, genericsettings.ppscatter_file_name + '.html', 'Scatter plots')
@@ -279,7 +295,7 @@ def save_single_functions_html(filename,
                                function_groups=None,
                                parentFileName=None,  # used only with HtmlPage.NON_SPECIFIED
                                header=None,  # used only with HtmlPage.NON_SPECIFIED
-                               caption=None):  # used only with HtmlPage.NON_SPECIFIED
+                               caption=None):  # used only with HtmlPage.NON_SPECIFIED and PPFIGCONS1 (dynamic caption setting)
 
     name = filename.split(os.sep)[-1]
     current_dir = os.path.dirname(os.path.realpath(filename))
@@ -292,11 +308,11 @@ def save_single_functions_html(filename,
         if function_groups is None:
             function_groups = OrderedDict([])
 
-        function_group = "nzall" if genericsettings.isNoisy else "noiselessall"
-        if htmlPage not in (HtmlPage.PPRLDMANY_BY_GROUP, HtmlPage.PPLOGLOSS):
-            temp_function_groups = OrderedDict([(function_group, 'All functions')])
-            temp_function_groups.update(function_groups)
-            function_groups = temp_function_groups
+        if not testbedsettings.current_testbed.name.startswith("bbob-constrained"):
+            function_group = "nzall" if genericsettings.isNoisy else "noiselessall"
+            if htmlPage not in (HtmlPage.PPRLDMANY_BY_GROUP, HtmlPage.PPLOGLOSS):
+                function_groups.update(
+                    OrderedDict([(function_group, 'All functions')]))  # append the noiselesall group at the end
 
         first_function_number = testbedsettings.current_testbed.first_function_number
         last_function_number = testbedsettings.current_testbed.last_function_number
@@ -309,7 +325,6 @@ def save_single_functions_html(filename,
             f.write("\n<H2> %s </H2>\n" % current_header)
             for function_number in range(first_function_number, last_function_number + 1):
                 f.write(add_image('ppscatter_f%03d%s.%s' % (function_number, add_to_names, extension), True))
-
             f.write(caption_string_format % '##bbobppscatterlegend##')
 
         elif htmlPage is HtmlPage.PPFIGS:
@@ -319,6 +334,22 @@ def save_single_functions_html(filename,
                 f.write(add_image('ppfigs_f%03d%s.%s' % (function_number, add_to_names, extension), True))
             f.write(caption_string_format % '##bbobppfigslegend##')
 
+        elif htmlPage is HtmlPage.PPFIGCONS:
+            current_header = 'Scaling of run "time" with number of constraints'
+            f.write("\n<H2> %s </H2>\n" % current_header)
+            for function_name in testbedsettings.current_testbed.func_cons_groups.keys():
+                for dimension in dimensions if dimensions is not None else [2, 3, 5, 10, 20, 40]:
+                    f.write(add_image('ppfigcons_%s_d%s%s.%s' % (function_name, dimension, add_to_names, extension), True))
+            f.write(caption_string_format % '##bbobppfigconslegend##')
+
+        elif htmlPage is HtmlPage.PPFIGCONS1:
+            current_header = 'Scaling of run "time" with number of constraints for selected targets'
+            f.write("\n<H2> %s </H2>\n" % current_header)
+            for function_name in testbedsettings.current_testbed.func_cons_groups.keys():
+                for dimension in dimensions if dimensions is not None else [2, 3, 5, 10, 20, 40]:
+                    f.write(add_image('ppfigcons1_%s_d%s%s.%s' % (function_name, dimension, add_to_names, extension), True))
+            # caption given by caption in this case
+
         elif htmlPage is HtmlPage.NON_SPECIFIED:
             current_header = header
             f.write("\n<H2> %s </H2>\n" % current_header)
@@ -457,7 +488,7 @@ def write_dimension_links(dimension, dimensions, index):
 
 
 def write_tables(f, caption_string_format, best_alg_exists, html_key, legend_key, dimensions):
-    current_header = 'Table showing the ERT in number of function evaluations'
+    current_header = 'Table showing the ERT in number of evaluations'
     if best_alg_exists:
         current_header += ' divided by the best ERT measured during BBOB-2009'
 
@@ -569,7 +600,8 @@ def plotUnifLogXMarkers(x, y, nbperdecade, logscale=False, **kwargs):
                                   np.log10 if logscale else None)
         marker_args = kwargs.copy()
         marker_args['drawstyle'] = 'default'
-        marker_args['linestyle'] = ''
+        marker_args['linestyle' if 'ls' not in marker_args else
+                    'ls'] = ''  # 'linestyle' cannot be mixed with 'ls' as used elsewhere
         marker_args['label'] = ''
         res2 = plt.plot(x2, y2, **marker_args)
         res.extend(res2)
@@ -737,7 +769,7 @@ def plot(dsList, _valuesOfInterest=(10, 1, 1e-1, 1e-2, 1e-3, 1e-5, 1e-8),
                 tmp = StrippedUpDS()
                 idxs = list(k + 1 for k in idx)
                 idxs.insert(0, 0)
-                tmp.evals = i.evals[:, np.r_[idxs]].copy()
+                tmp._evals = i._evals[:, np.r_[idxs]].copy()
                 tmp.maxevals = i.maxevals[np.ix_(idx)].copy()
                 res.setdefault(j, [])
                 res.get(j).append(tmp)
@@ -856,11 +888,13 @@ def get_plotting_styles(algorithms, only_foreground=False):
                 ppfigs_styles = {'marker': '',
                                  'color': format[0],
                                  'linestyle': format[1],
+                                 'zorder': -1.5,
                                  }
                 pprldmany_styles = {'marker': '',
                                     'label': '',
                                     'color': format[0],
                                     'linestyle': format[1],
+                                    'zorder': -1.5,
                                     }
                 plotting_styles.append(PlottingStyle(pprldmany_styles, ppfigs_styles, background_algorithms, True))
 
