@@ -27,18 +27,22 @@ from .toolsdivers import truncate_latex_command_file, print_done, diff_attr
 from .ppfig import Usage
 from .compall import ppfigs
 
+import matplotlib.pyplot as plt
+matplotlib.rcParams['pdf.fonttype'] = 42
+matplotlib.rcParams['ps.fonttype'] = 42
 matplotlib.use('Agg')  # To avoid window popup and use without X forwarding
 
 __all__ = ['main']
 
 # Used by getopt:
-shortoptlist = "hvo:"
-longoptlist = ["help", "output-dir=", "noisy", "noise-free",
+short_options = "hvo:"
+long_options = ["help", "output-dir=", "noisy", "noise-free",
                "tab-only", "fig-only", "rld-only", "no-rld-single-fcts",
                "verbose", "settings=", "conv",
                "expensive", "runlength-based",
                "los-only", "crafting-effort=", "pickle",
-               "sca-only", "no-svg"]
+               "sca-only", "no-svg",
+               "include-fonts"]
 # thereby, "los-only", "crafting-effort=", and "pickle" affect only rungeneric1
 # and "sca-only" only affects rungenericmany
 
@@ -53,7 +57,7 @@ def _split_short_opt_list(short_opt_list):
     tmp = short_opt_list[:]
     # split into logical elements: one-letter that could be followed by colon
     while tmp:
-        if len(tmp) > 1 and tmp[1] is ':':
+        if len(tmp) > 1 and tmp[1] == ':':
             res.add(tmp[0:2])
             tmp = tmp[2:]
         else:
@@ -194,6 +198,11 @@ def main(argv=None):
 
             prepares also convergence plots with median function values over time
 
+        --include-fonts
+
+            generated pdfs will have the fonts included (important for ACM style
+            LaTeX submissions)
+
 
     Exceptions raised:
 
@@ -221,8 +230,8 @@ def main(argv=None):
     also be presented as a list of strings.
 
     """
-    global shortoptlist
-    global longoptlist
+    # global shortoptlist
+    # global longoptlist
 
     if argv is None:
         argv = sys.argv[1:]
@@ -230,8 +239,8 @@ def main(argv=None):
         argv = argv.split()
     try:
         try:
-            opts, args = getopt.getopt(argv, shortoptlist,
-                                       longoptlist +
+            opts, args = getopt.getopt(argv, short_options,
+                                       long_options +
                                        ['include-single', 'in-a-hurry=', 'input-path='])
         except getopt.error as msg:
             raise Usage(msg)
@@ -248,11 +257,16 @@ def main(argv=None):
 
         # Process options
         shortoptlist = list("-" + i.rstrip(":")
-                            for i in _split_short_opt_list(shortoptlist))
-        shortoptlist.remove("-o")
-        longoptlist = list("--" + i.rstrip("=") for i in longoptlist)
-
+                            for i in _split_short_opt_list(short_options))
+        if "-o" in shortoptlist:
+            shortoptlist.remove("-o") # 2020/6/5: TODO: not sure why this is done
+        longoptlist = list("--" + i.rstrip("=") for i in long_options)
 
+        plt.rc("axes", **genericsettings.rcaxes)
+        plt.rc("xtick", **genericsettings.rctick)
+        plt.rc("ytick", **genericsettings.rctick)
+        plt.rc("font", **genericsettings.rcfont)
+        plt.rc("legend", **genericsettings.rclegend)
 
         genopts = []
         outputdir = genericsettings.outputdir
@@ -293,6 +307,11 @@ def main(argv=None):
                     genericsettings.inputCrE = float(a)
                 except ValueError:
                     raise Usage('Expect a valid float for flag crafting-effort.')
+            elif o == "--include-fonts":
+                plt.rc('pdf', fonttype=42)
+                plt.rcParams['pdf.fonttype'] = 42
+                matplotlib.rcParams['pdf.fonttype'] = 42
+                matplotlib.rcParams['ps.fonttype'] = 42
             elif o == "--tab-only":
                 genericsettings.isFig = False
                 genericsettings.isRLDistr = False
@@ -301,14 +320,21 @@ def main(argv=None):
                 genericsettings.isTab = False
                 genericsettings.isRLDistr = False
                 genericsettings.isLogLoss = False
+                genericsettings.isScatter = False
             elif o == "--rld-only":
                 genericsettings.isTab = False
                 genericsettings.isFig = False
                 genericsettings.isLogLoss = False
+                genericsettings.isScatter = False
             elif o == "--los-only":
                 genericsettings.isTab = False
                 genericsettings.isFig = False
                 genericsettings.isRLDistr = False
+            elif o == "--cons-only":
+                genericsettings.isTab = False
+                genericsettings.isFig = False
+                genericsettings.isRLDistr = False
+                genericsettings.isLogLoss = False
             else:
                 is_assigned = False
                 if o in longoptlist or o in shortoptlist:
@@ -360,7 +386,7 @@ def main(argv=None):
 
         update_background_algorithms(inputdir)
 
-        print('  Using:')
+        print('  Using %d data set%s:' % (len(args), 's' if len(args) > 1 else ''))
         for path in args:
             print('    %s' % path)
 
@@ -393,8 +419,8 @@ def main(argv=None):
                                         ['\\providecommand{\\numofalgs}{%d}' % len(args)]
                                         )
         toolsdivers.prepend_to_file(latex_commands_filename,
-                                    ['\\providecommand{\\cocoversion}{\\hspace{\\textwidth}\\scriptsize\\sffamily{}' +
-                                     '\\color{Gray}Data produced with COCO %s}' % (toolsdivers.get_version_label(None))]
+                                    ['\\providecommand{\\cocoversion}{{\\scriptsize\\sffamily{}' +
+                                     '\\color{Gray}Data produced with COCO %s}}' % (toolsdivers.get_version_label(None))]
                                     )
         toolsdivers.prepend_to_file(latex_commands_filename,
                                     ['\\providecommand{\\bbobecdfcaptionsinglefunctionssingledim}[1]{',
@@ -423,6 +449,8 @@ def main(argv=None):
             print('Setting changes in `cocopp.genericsettings` compared to default:')
             print(mess, end='')
 
+        plt.rcdefaults()
+
         print_done('ALL done')
         if genericsettings.interactive_mode:
             try:
