@@ -15,6 +15,8 @@ import shutil
 import subprocess
 import doctest
 
+from cocopp import archiving
+
 import matplotlib  # just to make sure the following is actually done first
 
 matplotlib.use('Agg')  # To avoid window popup and use without X forwarding
@@ -139,7 +141,7 @@ def retrieve_algorithm(data_path, folder_name, algorithm_name, file_name=None):
     replaced by `cocopp.archiving.get` or `cocopp.archiving.official_archives` or `cocopp.archives.all`)"""
     algorithm_file = join_path(data_path, file_name if file_name else algorithm_name)
     if not os.path.exists(algorithm_file):
-        data_url = 'http://coco.gforge.inria.fr/data-archive/%s/%s' % (folder_name, algorithm_name)
+        data_url = 'https://numbbo.github.io/data-archive/data-archive/%s/%s' % (folder_name, algorithm_name)
         urlretrieve(data_url, algorithm_file)
 
 
@@ -228,6 +230,17 @@ def delete_files(all_files=False):
         os.remove(join_path(cwd, 'acmcopyright.sty'))
         os.remove(join_path(cwd, 'bbob.bib'))
 
+def tmp_test_constrained(python, command):
+    url = "https://numbbo.github.io/cocopp-playground/bbob-constrained/testdata/bbob-constrained"
+    archive = archiving.get(url)
+    t0 = time.time()
+    data_path = archive.get("fmincon_consbbob")
+    print(python + command + data_path)
+    with InfolderGoneWithTheWind():
+        result = os.system(python + command + data_path)
+    print('**  subtest 19 finished in ', time.time() - t0, ' seconds')
+    assert result == 0, 'Test failed: rungeneric on one bbob-constrained algorithm.'
+
 
 def main(arguments):
     """these tests are executed when ``python cocopp`` is called.
@@ -249,6 +262,7 @@ def main(arguments):
 
     # old_data_path = ' ' + prepare_data(run_all_tests)
 
+
     command = ' cocopp --no-svg --settings=grayscale '  # TODO: grayscale has to go
 
     #copy_latex_templates()
@@ -275,6 +289,8 @@ def main(arguments):
     assert result == 0, 'Test failed: rungeneric on one bi-objective algorithm.'
     #run_latex_template("templateBIOBJarticle.tex", run_all_tests)
 
+    tmp_test_constrained(python, command)
+
     if run_all_tests:
         data_paths = data_archive_get([
                         'BIPOP-CMA-ES_hansen_noiseless',
@@ -284,6 +300,7 @@ def main(arguments):
                         'BFGS_ros_noiseless.tgz'])
         t0 = time.time()
         print(time.asctime())
+
         with InfolderGoneWithTheWind():
             result = os.system(python + command + data_paths)
         print('**  subtest 3 finished in ', time.time() - t0, ' seconds')
@@ -415,6 +432,18 @@ def main(arguments):
             print('**  subtest 18 finished in ', time.time() - t0, ' seconds')
             assert result == 0, 'Test failed: rungeneric on data from `bbob-biobj` and `bbob-biobj-ext` suite.'
 
+        with InfolderGoneWithTheWind():
+            t0 = time.time()
+            result = os.system("""python -c "import cocopp; cocopp.main('slsqp!'); cocopp.main('abc*')" """)
+            print('**  subtest 19 finished in ', time.time() - t0, ' seconds')
+            assert result == 0, 'Test failed: running postprocessing twice within same python session'
+
+        with InfolderGoneWithTheWind():
+            t0 = time.time()
+            result = os.system("""python -c "import cocopp; cocopp.main('constrained/2022/RandomSearch-5')" """)
+            print('**  subtest 20 finished in ', time.time() - t0, ' seconds')
+            assert result == 0, 'Test failed: running postprocessing on bbob-constrained data'
+
 
     print('launching doctest (it might be necessary to close a few pop up windows to finish)')
     t0 = time.time()
