@@ -187,7 +187,9 @@ def drawSP_from_dataset(data_set, ftarget, percentiles, samplesize=genericsettin
     
     The expected value of ``all_sampled_values_sorted`` is the expected
     runtime ERT, as obtained by ``data_set.detERT([ftarget])[0]``.
-    
+
+    Details: `samplesize` is adjusted (increased) such that it is zero when taken
+    modulo `data_set.nbRuns()`.
     """
     try:
         evals = data_set.detEvals([ftarget])[0]
@@ -195,7 +197,7 @@ def drawSP_from_dataset(data_set, ftarget, percentiles, samplesize=genericsettin
         print('drawSP_from_dataset expects a DataSet instance as first input, was: ' + str(type(data_set)))
         raise 
     nanidx = np.isnan(evals)
-    return drawSP(evals[~nanidx], data_set.maxevals[nanidx], percentiles, samplesize)
+    return drawSP(evals[~nanidx], data_set.maxevals[nanidx], percentiles, data_set.bootstrap_sample_size(samplesize))
 
 def drawSP_from_dataset_new(data_set, ftarget, dummy,
                             samplesize=genericsettings.simulated_runlength_bootstrap_sample_size):
@@ -692,9 +694,14 @@ def significancetest(entry0, entry1, targets):
 
     :returns: list of (z, p) for each target function values in
               input argument targets. z and p are values returned by the
-              ranksumtest method.
+              ranksumtest method. The z value is for `entry0` where a
+              larger value is better, because the test data are [-Df, 1/evals].
+
+    TODO: we would want to correct for imbalanced instances if the more
+    frequent instances are more different than the less frequent instances.
 
     """
+    balance_instances_saved, genericsettings.balance_instances = genericsettings.balance_instances, False
 
     bootstraps = False  # future extension
     res = []
@@ -753,7 +760,7 @@ def significancetest(entry0, entry1, targets):
                         else:
                             FE_umin = np.inf
                         # Determine the function values for FE_umin
-                        tmpfvalues = np.array([np.inf] * entry.nbRuns())
+                        prevline = np.array([np.inf] * (entry.funvals.shape[1] - 1))
                         for curline in entry.funvals:
                             # only works because the funvals are monotonous
                             if curline[0] > FE_umin:
@@ -779,7 +786,7 @@ def significancetest(entry0, entry1, targets):
                         FE.append(tmpfe)
                     FE_umin = min(FE)
 
-                    # Determine the function values for FE_umin
+                    # 2) determine the function values for FE_umin
                     fvalues = []
                     for j, entry in enumerate((entry0, entry1)):
                         prevline = np.array([np.inf] * entry.nbRuns())
@@ -793,6 +800,10 @@ def significancetest(entry0, entry1, targets):
         # 2. 3. 4. Collect data for the significance test:
         curdata = []  # current data 
         
+        try: fvalues
+        except NameError: pass
+        else:
+            f_offset = 1.01 * min((0, min(fvalues[0]), min(fvalues[1])))  # fix for negative fvalues (which are Df-values)
         for j, entry in enumerate((entry0, entry1)):
             tmp = evals[j][i].copy()
             idx = np.isnan(tmp)
@@ -800,8 +811,15 @@ def significancetest(entry0, entry1, targets):
             # was not a bool before: idx = np.isnan(tmp) + (tmp > FE_umin)
             tmp[idx == False] = np.power(tmp[idx == False], -1.)
             if idx.any():
-                tmp[idx] = -fvalues[j][idx]  # larger data is better
+                tmp[idx] = -fvalues[j][idx] + f_offset  # larger data is better
+                assert all(tmp[idx] <= 0), (
+                    "negative Df value(s) found ({}, offset={}) in DataSet {} in significance test line {}"
+                    " for target[{}] = {}. This is a bug and may lead to a wrong significance result."
+                    .format(-tmp[idx], f_offset, entry.info_str(targets), tmp, i, targets[i]))
             curdata.append(tmp)
+            if np.isnan(tmp).any():
+                warnings.warn("{} contains nan values in significance test line {} for target[{}] = {}"
+                              .format(entry.info_str(targets), tmp, i, targets[i]))
 
         z_and_p = ranksumtest(curdata[0], curdata[1])
         if isRefAlg:
@@ -813,46 +831,81 @@ def significancetest(entry0, entry1, targets):
             if not (erts[ibetter][i] <= erts[iworse][i] and  # inf are equal
                 (erts[ibetter][i] is np.inf or  # comparable data: only f-values are compared for significance (they are compared for the same #evals)
                  averageevals[ibetter][i] < averageevals[iworse][i])):  # better algorithm must not have larger effort, should this take into account FE_umin?
+            # remove significance if
+            #     either ert[better] > ert[worse] or
+            #     ert[better] is finite and average_evals[better] >= average_evals[worse] (e.g. the worse has no ert but only few evals)
+            # TODO (nitpicking): shouldn't the > in the first condition be a >= (same ert is not enough to be better unless both are inf)
+            #       and the >= in the second condition be a > (same average but more successes is enough)?
                 z_and_p = (z_and_p[0], 1.0)                  
 
         res.append(z_and_p)
 
+    genericsettings.balance_instances = balance_instances_saved
     return res
 
+def best_alg_indices(ert_ars=None, median_finalfunvals=None,
+                     datasets=None, targets=None):
+    """return the index of the most promising algorithm for each target.
+
+    `ert_ars` are the first criterion and computed from `datasets` and
+    `targets` if necessary. `median_finalfunvals` are needed when all
+    `ert_ars` are infinite for some target and computed from `datasets`
+    when necessary.
+
+    Lines in `ert_ars` contain ERTs for different targets, columns contain
+    ERTs for a single target from different algorithms.
+
+    Rationale: our primary (only) performance indicator is evals, hence we
+    use expected evals to determine the candidate for best algorithm. When
+    no evals are available, we have not run the experiment long enough (we
+    have no real measurement), then the best candidate is the one with the
+    best final f-values. The statistical test is later aligned to the
+    largest evals where all runs have a recorded value.
+    """
+    ert_ars = np.asarray([ds.detERT(targets) for ds in datasets] if ert_ars is None
+                         else ert_ars)
+    best_alg_idx = ert_ars.argsort(0)[0, :]  # index of best for each target value
+    for itarget, vals in enumerate(ert_ars.T):
+        if not np.any(np.isfinite(vals)):  # no single run reached the target
+            if median_finalfunvals is None:
+                median_finalfunvals = [np.median(ds.finalfunvals) for ds in datasets]
+            best_alg_idx[itarget] = np.argsort(median_finalfunvals)[0]
+    return best_alg_idx
+
 def significance_all_best_vs_other(datasets, targets, best_alg_idx=None):
     """:param datasets: is a list of DataSet from different algorithms, otherwise on the same function and dimension (which is not necessarily checked)
     :param targets: is a list of target values, 
     :param best_alg_idx:  for each target the best algorithm to be tested against the others 
     
-    returns a list of ``(z, p)`` tuples, each is the result for the ranksumtest 
-    for the respective target value in targets and the index list of best algorithm. 
-    
+    returns a list of ``(z, p)`` tuples, each results from the ranksum
+    tests for the respective target value for each algorithm against the
+    best algorithm defined from the index list. Each ``(z, p)`` is either
+    the first ``z`` when ``z >= 0`` (best algorithm is worse) in which case
+    ``p`` is set to 1, or, when ``z < 0``, it is the pair with the
+    largest observed ``p``.
     """ 
     if best_alg_idx is None:
-        erts = []
-        for ds in datasets:
-            erts.append(ds.detERT(targets))
-        best_alg_idx = np.array(erts).argsort(0)[0, :]  # indexed by target index
+        best_alg_idx = best_alg_indices(datasets=datasets, targets=targets)
         assert len(best_alg_idx) == len(targets)
-    elif 1 < 3:  # only for debugging
-        erts = []
-        for ds in datasets:
-            erts.append(ds.detERT(targets))
-          
-        best_alg_idx2 = np.array(erts).argsort(0)[0, :]  # indexed by target index
-        assert all(best_alg_idx2 == best_alg_idx)
         
     # significance test of best given algorithm against all others
     significance_versus_others = []  # indexed by target index
     assert len(best_alg_idx) == len(targets)
     if len(datasets) > 1:
         for itarget, target in enumerate(targets):
-            z_and_p = (0, 0)
-            for jalg in range(len(datasets)):
-                if jalg == best_alg_idx[itarget]:
+            z_and_p = None
+            for ialg in range(len(datasets)):
+                if ialg == best_alg_idx[itarget]:
                     continue
-                z_and_p2 = significancetest(datasets[jalg], datasets[best_alg_idx[itarget]], [target])[0]
-                if z_and_p2[1] > z_and_p[1]:  # look for strongest opponent, ie weakest p
+                z_and_p2 = significancetest(datasets[ialg],
+                                            datasets[best_alg_idx[itarget]], [target])[0]
+                if z_and_p2[0] >= 0:
+                    # found an algorithm that is better than best_alg_idx
+                    z_and_p = (z_and_p2[0], 1)
+                    break  # no need to check other algorithms
+                if z_and_p is None or z_and_p2[1] > z_and_p[1]:
+                    # when z was always < 0, ie all algorithms so far were indeed worse
+                    # then look for strongest opponent, ie weakest p (closest to 1)
                     z_and_p = z_and_p2 
             significance_versus_others.append(z_and_p)
     return significance_versus_others, best_alg_idx
